#include "myfndt.h"
#include "univopt.h"
#include "composed.h"
#include "rferr.h"

const int CUnivOptions::SectionLimit = 3;
const int CUnivOptions::SectionDelta = 3;

CUnivOptions::CUnivOptions():
	SectList(SectionLimit, SectionDelta),
#if defined(LIGHT_COMPILE)
    Unsupported(SectionLimit, SectionDelta),
#endif
    LastErrorMessage(NULL),
	ComposedKeys(0)
{
}

CUnivOptions::~CUnivOptions()
{
    if (LastErrorMessage)
        free(LastErrorMessage);
}

void CUnivOptions::Add(const char *Key, const char *Value)
{ 
    COptions<DString>::Add(Key, DString(Value)); 
}

char *CUnivOptions::UnknownKeyError(const char *Key)
{
    char *Message = OE_INVALKEY;

#if defined(LIGHT_COMPILE)
    DString KeyStr = Key;
    if (Unsupported.Find(&KeyStr) != INT_MAX)
        Message = OE_UNSUPPKEY;
#endif

    if (LastErrorMessage)
        return Message;
    LastErrorMessage = (char *)malloc(strlen(Message) + 1 + strlen(Key) + 1);
    if (!LastErrorMessage)
        return Message;
    
    strcpy(LastErrorMessage, Message);
    strcat(LastErrorMessage, " ");
    strcat(LastErrorMessage, Key);
    return LastErrorMessage;
}

DString CUnivOptions::FirstInvalidKey()
{
	return DString("");
}

void CUnivOptions::WatchSection(const char *SectName)
{
	SectList.Add(new DString(SectName));
}

short CUnivOptions::HasSection(const char *SectName)
{
	CObjPtr<DString> Test = new DString(SectName);
	return SectList.Find(Test) != INT_MAX;
}

DString CUnivOptions::GetKey(const char *Section,
	const char *Key) const
{
	if (!ComposedKeys)
		return DString(Key);
	else
	    return GetLongOptionKey(DString(Section), Key);
}

void CUnivOptions::ChangeWatchedSections(const CStringListValue &ToList)
{
     SectList.erase(SectList.begin(), SectList.end());

	CStringListValue::DIterator Itr = const_cast<CStringListValue &>(
                ToList).InitIterator();
	while (const_cast<CStringListValue &>(ToList).IsDereferenceable(Itr))
	{	SectList.Add(new DString(*Itr));
		++Itr;
	}
}

char *CUnivOptions::AllocMungedSectionName(char *InSection)
{
    return InSection;
}
